@@ -67,25 +67,26 @@ typedef DKG_Cheater STP_DKG_Cheater;
 /** Error code enum
  */
 typedef enum {
-  Err_OK = 0,
-  Err_ISize,
-  Err_OSize,
-  Err_OOB,
-  Err_Send,
-  Err_CheatersFound,
-  Err_CheatersFull,
-  Err_InvSessionID,
-  Err_Share,
-  Err_NoiseEncrypt,
-  Err_NoiseDecrypt,
-  Err_HMac,
-  Err_Index,
-  Err_NoSubVSPSFail,
-  Err_NotEnoughDealers,
-  Err_TooManyCheaters,
-  Err_DKGFinish,
-  Err_BroadcastEnv = 32,
-  Err_Env = 64
+  STP_DKG_Err_OK = 0,
+  STP_DKG_Err_ISize,
+  STP_DKG_Err_OSize,
+  STP_DKG_Err_OOB,
+  STP_DKG_Err_Send,
+  STP_DKG_Err_CheatersFound,
+  STP_DKG_Err_CheatersFull,
+  STP_DKG_Err_InvSessionID,
+  STP_DKG_Err_Share,
+  STP_DKG_Err_Noise,
+  STP_DKG_Err_NoiseEncrypt,
+  STP_DKG_Err_NoiseDecrypt,
+  STP_DKG_Err_HMac,
+  STP_DKG_Err_Index,
+  STP_DKG_Err_NoSubVSPSFail,
+  STP_DKG_Err_NotEnoughDealers,
+  STP_DKG_Err_TooManyCheaters,
+  STP_DKG_Err_DKGFinish,
+  STP_DKG_Err_BroadcastEnv = 32,
+  STP_DKG_Err_Env = 64
 } STP_DKG_Err;
 
 /** Steps of the STP enum
@@ -415,6 +416,10 @@ const uint8_t* stp_dkg_peerstate_lt_sk(const STP_DKG_PeerState *ctx);
     The size of the final share is TOPRF_Share_BYTES * 2
  */
 const uint8_t* stp_dkg_peerstate_share(const STP_DKG_PeerState *ctx);
+
+/** Getter for the commitments of the generated shares result of this protocol
+ */
+const uint8_t* stp_dkg_peerstate_commitments(const STP_DKG_PeerState *ctx);
 /** Getter for the current step of the Peer State
  */
 int stp_dkg_peerstate_step(const STP_DKG_PeerState *ctx);
@@ -435,6 +440,8 @@ int stp_dkg_peerstate_step(const STP_DKG_PeerState *ctx);
 
     @param [in] peer_lt_sk: the long-term private signing key of the peer.
 
+    @param [in] noise_sks: the long-term noise secret key of the peer.
+
     @param [in] msg0: the initiating msg sent from the TP after the TP
                 run tpdkg_tp_start().
 
@@ -447,12 +454,13 @@ int stp_dkg_peerstate_step(const STP_DKG_PeerState *ctx);
 STP_DKG_Err stp_dkg_start_peer(STP_DKG_PeerState *ctx,
                                const uint64_t ts_epsilon,
                                const uint8_t lt_sk[crypto_sign_SECRETKEYBYTES],
+                               const uint8_t noise_sks[crypto_scalarmult_SCALARBYTES],
                                const STP_DKG_Message *msg0,
                                uint8_t stp_ltpk[crypto_sign_PUBLICKEYBYTES]);
 
 /** This function sets all the variable sized buffers in the STP_DKG_PeerState structure.
 
-  The buffer sizes depend on the n and t parameters to the DKG tese
+  The buffer sizes depend on the n and t parameters to the DKG these
   are announced by the STP in msg0, which is an input to the
   tpdkg_start_peer() function, after this tpdkg_start_peer() function
   the peerstate is initialized and can be used to find out the n and t
@@ -572,10 +580,10 @@ size_t stp_dkg_stp_output_size(const STP_DKG_STPState *ctx);
    and stp_dkg_stp_output_size():
 
    @code
-    uint8_t tp_out[stp_dkg_tp_output_size(&tp)];
-    uint8_t tp_in[stp_dkg_tp_input_size(&tp)];
-    recv(socket, tp_in, sizeof(tp_in));
-    ret = stp_dkg_stp_next(&tp, tp_in, sizeof(tp_in), tp_out, sizeof tp_out);
+    uint8_t stp_out[stp_dkg_stp_output_size(&tp)];
+    uint8_t stp_in[stp_dkg_stp_input_size(&tp)];
+    recv(socket, stp_in, sizeof(stp_in));
+    ret = stp_dkg_stp_next(&tp, stp_in, sizeof(stp_in), stp_out, sizeof stp_out);
    @endcode
  */
 int stp_dkg_stp_next(STP_DKG_STPState *ctx, const uint8_t *input, const size_t input_len, uint8_t *output, const size_t output_len);
@@ -678,7 +686,7 @@ size_t stp_dkg_peer_output_size(const STP_DKG_PeerState *ctx);
    @code
    uint8_t peers_out[stp_dkg_peer_output_size(&peer)];
 
-   uint8_t peer_in[stp_dkg_peer_input_size(peers)];
+   uint8_t peer_in[stp_dkg_peer_input_size(&peer)];
    recv(socket, peer_in, sizeof(peer_in));
    ret = stp_dkg_peer_next(&peer,
                            peer_in, sizeof(peer_in),
@@ -689,7 +697,7 @@ int stp_dkg_peer_next(STP_DKG_PeerState *ctx, const uint8_t *input, const size_t
 
 /**
    This function checks if the protocol has finished for the peer or
-   more stp_dk_peer_next() calls are necessary.
+   more stp_dkg_peer_next() calls are necessary.
 
    @return 1 if more steps outstanding
  */
