@@ -169,15 +169,15 @@ int tpdkg_peerstate_step(const TP_DKG_PeerState *ctx);
     be used. The following variables are useful and can be used by
     users of this API:
 
-    @var TP_DKG_PeerState:n This field contains the value N,
+    @var TP_DKG_TPState:n This field contains the value N,
          specifying the total number of peers participating in this
          protocol.
 
-    @var TP_DKG_PeerState:t This field contains the value T,
+    @var TP_DKG_TPState:t This field contains the value T,
          specifying the threshold necessary to use shared secret
          generated by this DKG.
 
-    @var TP_DKG_PeerState:cheaters This field contains a list of
+    @var TP_DKG_TPState:cheaters This field contains a list of
          cheaters and protocol violators at the end of a failed
          protocol run.
 
@@ -206,11 +206,23 @@ typedef struct {
   crypto_generichash_state transcript;
 } TP_DKG_TPState;
 
+/** getter for the size of a TP_DKG_TPState struct
+ */
 size_t tpdkg_tpstate_size(void);
+/** getter for the value N from a TP_DKG_TPState context
+ */
 uint8_t tpdkg_tpstate_n(const TP_DKG_TPState *ctx);
+/** getter for the value T from a TP_DKG_TPState context
+ */
 uint8_t tpdkg_tpstate_t(const TP_DKG_TPState *ctx);
+/** getter for the number of cheater encountered from a TP_DKG_TPState context
+ */
 size_t tpdkg_tpstate_cheater_len(const TP_DKG_TPState *ctx);
+/** getter for the sessionid from a TP_DKG_TPState context
+ */
 const uint8_t* tpdkg_tpstate_sessionid(const TP_DKG_TPState *ctx);
+/** getter for the current step of a TP_DKG_TPState context
+ */
 int tpdkg_tpstate_step(const TP_DKG_TPState *ctx);
 
 /*
